/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.sys.modular.department.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.sys.modular.department.entity.SysDepartment;
import vip.xiaonuo.sys.modular.department.mapper.SysDepartmentMapper;
import vip.xiaonuo.sys.modular.department.param.DepartmentAddParam;
import vip.xiaonuo.sys.modular.department.param.DepartmentEditParam;
import vip.xiaonuo.sys.modular.department.param.DepartmentIdParam;
import vip.xiaonuo.sys.modular.department.param.DepartmentPageParam;
import vip.xiaonuo.sys.modular.department.service.SysDepartmentService;

import java.util.List;

/**
 * 部门Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/22 19:30
 **/
@Service
public class SysDepartmentServiceImpl extends ServiceImpl<SysDepartmentMapper, SysDepartment> implements SysDepartmentService {

    @Override
    public Page<SysDepartment> page(DepartmentPageParam departmentPageParam) {
        QueryWrapper<SysDepartment> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isAllNotEmpty(departmentPageParam.getSortField(), departmentPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(departmentPageParam.getSortOrder());
            queryWrapper.orderBy(true, departmentPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(departmentPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(SysDepartment::getSortCode);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(DepartmentAddParam departmentAddParam) {
        SysDepartment department = BeanUtil.toBean(departmentAddParam, SysDepartment.class);
        this.save(department);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(DepartmentEditParam departmentEditParam) {
        SysDepartment department = this.queryEntity(departmentEditParam.getId());
        BeanUtil.copyProperties(departmentEditParam, department);
        this.updateById(department);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<DepartmentIdParam> departmentIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(departmentIdParamList, DepartmentIdParam::getId));
    }

    @Override
    public SysDepartment detail(DepartmentIdParam departmentIdParam) {
        return this.queryEntity(departmentIdParam.getId());
    }

    @Override
    public SysDepartment queryEntity(String id) {
        SysDepartment department = this.getById(id);
        if(ObjectUtil.isEmpty(department)) {
            throw new CommonException("部门不存在，id值为：{}", id);
        }
        return department;
    }
}
