/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.aiDevice.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.ai.aiDevice.entity.AiDevice;
import vip.xiaonuo.modular.ai.aiDevice.entity.vo.AiDeviceCountVo;
import vip.xiaonuo.modular.ai.aiDevice.mapper.AiDeviceMapper;
import vip.xiaonuo.modular.ai.aiDevice.param.AiDeviceAddParam;
import vip.xiaonuo.modular.ai.aiDevice.param.AiDeviceEditParam;
import vip.xiaonuo.modular.ai.aiDevice.param.AiDeviceIdParam;
import vip.xiaonuo.modular.ai.aiDevice.param.AiDevicePageParam;
import vip.xiaonuo.modular.ai.aiDevice.service.AiDeviceService;
import vip.xiaonuo.modular.energy.energyDevice.entity.EnergyDevice;
import vip.xiaonuo.modular.energy.energyDevice.entity.vo.EnergyDeviceCountVo;

import java.text.NumberFormat;
import java.util.List;

/**
 * 巡检设备Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/06 11:20
 **/
@Service
public class AiDeviceServiceImpl extends ServiceImpl<AiDeviceMapper, AiDevice> implements AiDeviceService {

    @Override
    public Page<AiDevice> page(AiDevicePageParam aiDevicePageParam) {
        QueryWrapper<AiDevice> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(aiDevicePageParam.getDeviceTypeId())) {
            queryWrapper.lambda().eq(AiDevice::getDeviceTypeId, aiDevicePageParam.getDeviceTypeId());
        }
        if(ObjectUtil.isNotEmpty(aiDevicePageParam.getInstallPosition())) {
            queryWrapper.lambda().eq(AiDevice::getInstallPosition, aiDevicePageParam.getInstallPosition());
        }
        if(ObjectUtil.isAllNotEmpty(aiDevicePageParam.getSortField(), aiDevicePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(aiDevicePageParam.getSortOrder());
            queryWrapper.orderBy(true, aiDevicePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(aiDevicePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(AiDevice::getId);
        }
        queryWrapper.lambda().eq(AiDevice::getOrgId,aiDevicePageParam.getOrgId());
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(AiDeviceAddParam aiDeviceAddParam) {
        AiDevice aiDevice = BeanUtil.toBean(aiDeviceAddParam, AiDevice.class);
        this.save(aiDevice);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(AiDeviceEditParam aiDeviceEditParam) {
        AiDevice aiDevice = this.queryEntity(aiDeviceEditParam.getId());
        BeanUtil.copyProperties(aiDeviceEditParam, aiDevice);
        this.updateById(aiDevice);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<AiDeviceIdParam> aiDeviceIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(aiDeviceIdParamList, AiDeviceIdParam::getId));
    }

    @Override
    public AiDevice detail(AiDeviceIdParam aiDeviceIdParam) {
        return this.queryEntity(aiDeviceIdParam.getId());
    }

    @Override
    public AiDevice queryEntity(String id) {
        AiDevice aiDevice = this.getById(id);
        if(ObjectUtil.isEmpty(aiDevice)) {
            throw new CommonException("巡检设备不存在，id值为：{}", id);
        }
        return aiDevice;
    }

    @Override
    public AiDeviceCountVo deviceCount(OrgIdParam orgIdParam) {
        AiDeviceCountVo aiDeviceCountVo = new AiDeviceCountVo();
        String orgId = orgIdParam.getOrgId();
        long deviceNum = this.count(new QueryWrapper<AiDevice>().lambda().eq(AiDevice::getOrgId, orgId));
        if (deviceNum == 0){
            aiDeviceCountVo.setDeviceNum(0L);
            aiDeviceCountVo.setOnDeviceNum(0L);
            aiDeviceCountVo.setOffDeviceNum(0L);
            aiDeviceCountVo.setOnlineRate("0%");
        }else {
            aiDeviceCountVo.setDeviceNum(deviceNum);
            long onDeviceNum = this.count(new QueryWrapper<AiDevice>().lambda().eq(AiDevice::getStatus,"1").eq(AiDevice::getOrgId, orgId));
            aiDeviceCountVo.setOnDeviceNum(onDeviceNum);
            long offDeviceNum = this.count(new QueryWrapper<AiDevice>().lambda().eq(AiDevice::getStatus,"0").eq(AiDevice::getOrgId, orgId));
            aiDeviceCountVo.setOffDeviceNum(offDeviceNum);
            //创建一个数值格式化对象
            NumberFormat numberFormat = NumberFormat.getInstance();
            //设置精确到小数点后两位
            numberFormat.setMaximumFractionDigits(2);
            System.out.println("onDeviceNum"+onDeviceNum);
            System.out.println("deviceNum"+deviceNum);
            String onlineRate = numberFormat.format((float)onDeviceNum / (float) deviceNum* 100) + "%";
            System.out.println("onlineRate"+onlineRate);
            aiDeviceCountVo.setOnlineRate(onlineRate);
        }
        return aiDeviceCountVo;
    }
}
