/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.dirtyCapture.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.ai.dirtyCapture.entity.DirtyCapture;
import vip.xiaonuo.modular.ai.dirtyCapture.mapper.DirtyCaptureMapper;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCaptureAddParam;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCaptureEditParam;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCaptureIdParam;
import vip.xiaonuo.modular.ai.dirtyCapture.param.DirtyCapturePageParam;
import vip.xiaonuo.modular.ai.dirtyCapture.service.DirtyCaptureService;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEvents;
import vip.xiaonuo.modular.ai.followEvents.service.FollowEventsService;
import vip.xiaonuo.modular.ai.personCapture.entity.PersonCapture;

import java.util.List;

/**
 * 设备镜头脏污Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/06 11:30
 **/
@Service
public class DirtyCaptureServiceImpl extends ServiceImpl<DirtyCaptureMapper, DirtyCapture> implements DirtyCaptureService {

    @Autowired
    private FollowEventsService followEventsService;

    @Override
    public Page<DirtyCapture> page(DirtyCapturePageParam dirtyCapturePageParam) {
        QueryWrapper<DirtyCapture> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(dirtyCapturePageParam.getStartCapTime()) && ObjectUtil.isNotEmpty(dirtyCapturePageParam.getEndCapTime())) {
            queryWrapper.lambda().between(DirtyCapture::getCapTime, dirtyCapturePageParam.getStartCapTime(), dirtyCapturePageParam.getEndCapTime());
        }
        if(ObjectUtil.isAllNotEmpty(dirtyCapturePageParam.getSortField(), dirtyCapturePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(dirtyCapturePageParam.getSortOrder());
            queryWrapper.orderBy(true, dirtyCapturePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(dirtyCapturePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByDesc(DirtyCapture::getCapTime);
        }
        queryWrapper.lambda().eq(DirtyCapture::getOrgId,dirtyCapturePageParam.getOrgId());
        Page<DirtyCapture> page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        List<DirtyCapture> records = page.getRecords();
        for (DirtyCapture record : records) {
            long count = followEventsService.count(new QueryWrapper<FollowEvents>().lambda()
                    .eq(FollowEvents::getEventId, record.getId())
                    .eq(FollowEvents::getEventType, 4).eq(FollowEvents::getUserId, StpLoginUserUtil.getLoginUser().getId()));
            if (count > 0){
                record.setIsFollow(1);
            }else {
                record.setIsFollow(0);
            }
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(DirtyCaptureAddParam dirtyCaptureAddParam) {
        DirtyCapture dirtyCapture = BeanUtil.toBean(dirtyCaptureAddParam, DirtyCapture.class);
        this.save(dirtyCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(DirtyCaptureEditParam dirtyCaptureEditParam) {
        DirtyCapture dirtyCapture = this.queryEntity(dirtyCaptureEditParam.getId());
        BeanUtil.copyProperties(dirtyCaptureEditParam, dirtyCapture);
        this.updateById(dirtyCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<DirtyCaptureIdParam> dirtyCaptureIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(dirtyCaptureIdParamList, DirtyCaptureIdParam::getId));
    }

    @Override
    public DirtyCapture detail(DirtyCaptureIdParam dirtyCaptureIdParam) {
        return this.queryEntity(dirtyCaptureIdParam.getId());
    }

    @Override
    public DirtyCapture queryEntity(String id) {
        DirtyCapture dirtyCapture = this.getById(id);
        if(ObjectUtil.isEmpty(dirtyCapture)) {
            throw new CommonException("设备镜头脏污不存在，id值为：{}", id);
        }
        return dirtyCapture;
    }
}
