/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.followEvents.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import io.swagger.models.auth.In;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEvents;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEventsVo;
import vip.xiaonuo.modular.ai.followEvents.mapper.FollowEventsMapper;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsAddParam;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsEditParam;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsIdParam;
import vip.xiaonuo.modular.ai.followEvents.param.FollowEventsPageParam;
import vip.xiaonuo.modular.ai.followEvents.service.FollowEventsService;

import java.util.List;

/**
 * 事件关注Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/24 10:50
 **/
@Service
public class FollowEventsServiceImpl extends ServiceImpl<FollowEventsMapper, FollowEvents> implements FollowEventsService {

    @Autowired
    private FollowEventsMapper followEventsMapper;

    @Override
    public Page<FollowEventsVo> page(FollowEventsPageParam followEventsPageParam) {
        String id = StpLoginUserUtil.getLoginUser().getId();
        Integer eventType = followEventsPageParam.getEventType();
        return followEventsMapper.page(CommonPageRequest.defaultPage(), id,eventType);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(FollowEventsAddParam followEventsAddParam) {
        FollowEvents followEvents = BeanUtil.toBean(followEventsAddParam, FollowEvents.class);
        followEvents.setUserId(StpLoginUserUtil.getLoginUser().getId());
        this.save(followEvents);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(FollowEventsEditParam followEventsEditParam) {
        FollowEvents followEvents = this.queryEntity(followEventsEditParam.getId());
        BeanUtil.copyProperties(followEventsEditParam, followEvents);
        this.updateById(followEvents);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<FollowEventsIdParam> followEventsIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(followEventsIdParamList, FollowEventsIdParam::getId));
    }

    @Override
    public FollowEvents detail(FollowEventsIdParam followEventsIdParam) {
        return this.queryEntity(followEventsIdParam.getId());
    }

    @Override
    public FollowEvents queryEntity(String id) {
        FollowEvents followEvents = this.getById(id);
        if(ObjectUtil.isEmpty(followEvents)) {
            throw new CommonException("事件关注不存在，id值为：{}", id);
        }
        return followEvents;
    }

    @Override
    public void cancel(FollowEventsAddParam followEventsAddParam) {
        String userId = StpLoginUserUtil.getLoginUser().getId();
        this.remove(new QueryWrapper<FollowEvents>().lambda()
                .eq(FollowEvents::getUserId,userId)
                .eq(FollowEvents::getEventId,followEventsAddParam.getEventId())
                .eq(FollowEvents::getEventType,followEventsAddParam.getEventType()));
    }
}
