/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.personCapture.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.auth.core.util.StpLoginUserUtil;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.core.param.OrgIdParam;
import vip.xiaonuo.modular.ai.carCapture.entity.CarCapture;
import vip.xiaonuo.modular.ai.carCapture.entity.vo.CarCaptureHourVo;
import vip.xiaonuo.modular.ai.carCapture.entity.vo.CarCapturePastVo;
import vip.xiaonuo.modular.ai.followEvents.entity.FollowEvents;
import vip.xiaonuo.modular.ai.followEvents.service.FollowEventsService;
import vip.xiaonuo.modular.ai.personCapture.entity.PersonCapture;
import vip.xiaonuo.modular.ai.personCapture.entity.vo.PersonCaptureHourVo;
import vip.xiaonuo.modular.ai.personCapture.entity.vo.PersonCapturePastVo;
import vip.xiaonuo.modular.ai.personCapture.mapper.PersonCaptureMapper;
import vip.xiaonuo.modular.ai.personCapture.param.PersonCaptureAddParam;
import vip.xiaonuo.modular.ai.personCapture.param.PersonCaptureEditParam;
import vip.xiaonuo.modular.ai.personCapture.param.PersonCaptureIdParam;
import vip.xiaonuo.modular.ai.personCapture.param.PersonCapturePageParam;
import vip.xiaonuo.modular.ai.personCapture.service.PersonCaptureService;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.List;

/**
 * 人员识别Service接口实现类
 *
 * @author newspaper
 * @date  2024/01/06 13:56
 **/
@Service
public class PersonCaptureServiceImpl extends ServiceImpl<PersonCaptureMapper, PersonCapture> implements PersonCaptureService {

    @Autowired
    private PersonCaptureMapper personCaptureMapper;

    @Autowired
    private FollowEventsService followEventsService;

    @Override
    public Page<PersonCapture> page(PersonCapturePageParam personCapturePageParam) {
        QueryWrapper<PersonCapture> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(personCapturePageParam.getStartCapTime()) && ObjectUtil.isNotEmpty(personCapturePageParam.getEndCapTime())) {
            queryWrapper.lambda().between(PersonCapture::getCapTime, personCapturePageParam.getStartCapTime(), personCapturePageParam.getEndCapTime());
        }
        if(ObjectUtil.isNotEmpty(personCapturePageParam.getPersonType())) {
            queryWrapper.lambda().eq(PersonCapture::getPersonType, personCapturePageParam.getPersonType());
        }
        if(ObjectUtil.isNotEmpty(personCapturePageParam.getCapResult())) {
            queryWrapper.lambda().eq(PersonCapture::getCapResult, personCapturePageParam.getCapResult());
        }
        if(ObjectUtil.isAllNotEmpty(personCapturePageParam.getSortField(), personCapturePageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(personCapturePageParam.getSortOrder());
            queryWrapper.orderBy(true, personCapturePageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(personCapturePageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByDesc(PersonCapture::getCapTime);
        }
        queryWrapper.lambda().eq(PersonCapture::getOrgId,personCapturePageParam.getOrgId());
        Page<PersonCapture> page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        List<PersonCapture> records = page.getRecords();
        for (PersonCapture record : records) {
            long count = followEventsService.count(new QueryWrapper<FollowEvents>().lambda()
                    .eq(FollowEvents::getEventId, record.getId())
                    .eq(FollowEvents::getEventType, 1).eq(FollowEvents::getUserId, StpLoginUserUtil.getLoginUser().getId()));
            if (count > 0){
                record.setIsFollow(1);
            }else {
                record.setIsFollow(0);
            }
        }
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PersonCaptureAddParam personCaptureAddParam) {
        PersonCapture personCapture = BeanUtil.toBean(personCaptureAddParam, PersonCapture.class);
        this.save(personCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PersonCaptureEditParam personCaptureEditParam) {
        PersonCapture personCapture = this.queryEntity(personCaptureEditParam.getId());
        BeanUtil.copyProperties(personCaptureEditParam, personCapture);
        this.updateById(personCapture);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PersonCaptureIdParam> personCaptureIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(personCaptureIdParamList, PersonCaptureIdParam::getId));
    }

    @Override
    public PersonCapture detail(PersonCaptureIdParam personCaptureIdParam) {
        return this.queryEntity(personCaptureIdParam.getId());
    }

    @Override
    public PersonCapture queryEntity(String id) {
        PersonCapture personCapture = this.getById(id);
        if(ObjectUtil.isEmpty(personCapture)) {
            throw new CommonException("人员识别不存在，id值为：{}", id);
        }
        return personCapture;
    }

    @Override
    public PersonCapturePastVo getPastCapture(OrgIdParam orgIdParam) {
        PersonCapturePastVo personCapturePastVo = new PersonCapturePastVo();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");
        // 获取当前时间
        LocalDateTime currentTime = LocalDateTime.now();
        // 计算12小时前的时间
        LocalDateTime twelveHoursAgo = currentTime.minusHours(12);
        String twelveHoursAgoFormatted = twelveHoursAgo.format(formatter);
        long count = this.count(new QueryWrapper<PersonCapture>().lambda().eq(PersonCapture::getOrgId, orgIdParam.getOrgId()).ge(PersonCapture::getCapTime, twelveHoursAgoFormatted));
        personCapturePastVo.setTotal(count);
        List<PersonCaptureHourVo> carCaptureHourVos = personCaptureMapper.listPast(orgIdParam.getOrgId());
        personCapturePastVo.setList(carCaptureHourVos);
        return personCapturePastVo;
    }
}
