/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.ai.taskInspection.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.ai.taskInspection.entity.TaskInspection;
import vip.xiaonuo.modular.ai.taskInspection.entity.vo.TaskInspectionDetailVo;
import vip.xiaonuo.modular.ai.taskInspection.entity.vo.TaskInspectionImageVo;
import vip.xiaonuo.modular.ai.taskInspection.entity.vo.TaskInspectionNumVo;
import vip.xiaonuo.modular.ai.taskInspection.entity.vo.TaskInspectionPageVo;
import vip.xiaonuo.modular.ai.taskInspection.param.TaskInspectionAddParam;
import vip.xiaonuo.modular.ai.taskInspection.param.TaskInspectionEditParam;
import vip.xiaonuo.modular.ai.taskInspection.param.TaskInspectionIdParam;
import vip.xiaonuo.modular.ai.taskInspection.param.TaskInspectionPageParam;
import vip.xiaonuo.modular.ai.taskInspection.service.TaskInspectionService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.text.ParseException;

/**
 * 任务巡检控制器
 *
 * @author newspaper
 * @date  2024/01/08 09:21
 */
@Api(tags = "任务巡检控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class TaskInspectionController {

    @Resource
    private TaskInspectionService taskInspectionService;

    /**
     * 获取任务巡检分页
     *
     * @author newspaper
     * @date  2024/01/08 09:21
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取任务巡检分页")
    @GetMapping("/ai/taskInspection/page")
    public CommonResult<Page<TaskInspectionPageVo>> page(TaskInspectionPageParam taskInspectionPageParam) {
        return CommonResult.data(taskInspectionService.page(taskInspectionPageParam));
    }

    /**
     * 提交任务巡检
     *
     * @author newspaper
     * @date  2024/01/08 09:21
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("提交任务巡检")
    @CommonLog("提交任务巡检")
    @PostMapping("/ai/taskInspection/edit")
    public CommonResult<String> edit(@RequestParam @ApiParam(value = "任务巡检id") String id,
                                     @RequestParam @ApiParam(value = "巡检时间") String inspectTime,
                                     @RequestParam @ApiParam(value = "巡检人员id") String inspectPersonId,
                                     @RequestParam @ApiParam(value = "问题描述") String description,
                                     @RequestParam @ApiParam(value = "今日不再提醒 ") String noRemind,
                                     @RequestParam(value = "img1") @ApiParam(value = "图片1") MultipartFile img1,
                                     @RequestParam(value = "img2", required = false) @ApiParam(value = "图片2") MultipartFile img2,
                                     @RequestParam(value = "img3", required = false) @ApiParam(value = "图片3") MultipartFile img3) throws IOException, ParseException {
        taskInspectionService.edit(id,inspectTime,inspectPersonId,description,noRemind,img1,img2,img3);
        return CommonResult.ok();
    }

    /**
     * 删除任务巡检
     *
     * @author newspaper
     * @date  2024/01/08 09:21
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("删除任务巡检")
    @CommonLog("删除任务巡检")
    @PostMapping("/ai/taskInspection/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<TaskInspectionIdParam> taskInspectionIdParamList) {
        taskInspectionService.delete(taskInspectionIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取任务巡检详情
     *
     * @author newspaper
     * @date  2024/01/08 09:21
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("获取任务巡检详情")
    @GetMapping("/ai/taskInspection/detail")
    public CommonResult<TaskInspectionImageVo> detail(TaskInspectionIdParam taskInspectionIdParam) {
        return CommonResult.data(taskInspectionService.detail(taskInspectionIdParam));
    }

    /**
     * 获取巡检数量
     *
     * @author newspaper
     * @date  2024/01/08 09:21
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取巡检数量")
    @GetMapping("/ai/taskInspection/getNum")
    public CommonResult<TaskInspectionNumVo> getNum() {
        return CommonResult.data(taskInspectionService.getNum());
    }
}
