/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigbackfat.controller;

import cn.dev33.satoken.annotation.SaCheckPermission;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.modular.bi.pigbackfat.entity.PigBackfat;
import vip.xiaonuo.modular.bi.pigbackfat.param.PigBackfatAddParam;
import vip.xiaonuo.modular.bi.pigbackfat.param.PigBackfatEditParam;
import vip.xiaonuo.modular.bi.pigbackfat.param.PigBackfatIdParam;
import vip.xiaonuo.modular.bi.pigbackfat.param.PigBackfatPageParam;
import vip.xiaonuo.modular.bi.pigbackfat.service.PigBackfatService;

import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;

/**
 * 背膘测定控制器
 *
 * @author wang
 * @date  2023/12/07 17:54
 */
@Api(tags = "背膘测定控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PigBackfatController {

    @Resource
    private PigBackfatService pigBackfatService;

    /**
     * 获取背膘测定分页
     *
     * @author wang
     * @date  2023/12/07 17:54
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取背膘测定分页")
    @GetMapping("/bi/pigbackfat/page")
    public CommonResult<Page<PigBackfat>> page(PigBackfatPageParam pigBackfatPageParam) {
        return CommonResult.data(pigBackfatService.page(pigBackfatPageParam));
    }

    /**
     * 添加背膘测定
     *
     * @author wang
     * @date  2023/12/07 17:54
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加背膘测定")
    @CommonLog("添加背膘测定")
    @PostMapping("/bi/pigbackfat/add")
    public CommonResult<String> add(@RequestBody @Valid PigBackfatAddParam pigBackfatAddParam) {
        pigBackfatService.add(pigBackfatAddParam);
        return CommonResult.ok();
    }

    /**
     * 编辑背膘测定
     *
     * @author wang
     * @date  2023/12/07 17:54
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑背膘测定")
    @CommonLog("编辑背膘测定")
    @PostMapping("/bi/pigbackfat/edit")
    public CommonResult<String> edit(@RequestBody @Valid PigBackfatEditParam pigBackfatEditParam) {
        pigBackfatService.edit(pigBackfatEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除背膘测定
     *
     * @author wang
     * @date  2023/12/07 17:54
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除背膘测定")
    @CommonLog("删除背膘测定")
    @PostMapping("/bi/pigbackfat/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<PigBackfatIdParam> pigBackfatIdParamList) {
        pigBackfatService.delete(pigBackfatIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取背膘测定详情
     *
     * @author wang
     * @date  2023/12/07 17:54
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取背膘测定详情")
    @GetMapping("/bi/pigbackfat/detail")
    public CommonResult<PigBackfat> detail(@Valid PigBackfatIdParam pigBackfatIdParam) {
        return CommonResult.data(pigBackfatService.detail(pigBackfatIdParam));
    }
}
