/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigdeath.controller;

import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.transaction.interceptor.TransactionAspectSupport;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import vip.xiaonuo.common.annotation.CommonLog;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.common.pojo.CommonValidList;
import vip.xiaonuo.common.util.UploadImage;
import vip.xiaonuo.modular.bi.pigdeath.entity.PigDeath;
import vip.xiaonuo.modular.bi.pigdeath.param.PigDeathAddParam;
import vip.xiaonuo.modular.bi.pigdeath.param.PigDeathEditParam;
import vip.xiaonuo.modular.bi.pigdeath.param.PigDeathIdParam;
import vip.xiaonuo.modular.bi.pigdeath.param.PigDeathPageParam;
import vip.xiaonuo.modular.bi.pigdeath.service.PigDeathService;


import javax.annotation.Resource;
import javax.validation.Valid;
import javax.validation.constraints.NotEmpty;
import java.io.IOException;
import java.util.Date;

/**
 * 死亡信息控制器
 *
 * @author wang
 * @date  2023/12/07 17:36
 */
@Api(tags = "死亡信息控制器")
@ApiSupport(author = "SNOWY_TEAM", order = 1)
@RestController
@Validated
public class PigDeathController {

    @Resource
    private PigDeathService pigDeathService;
    @Resource
    private UploadImage uploadImage;

    /**
     * 获取死亡信息分页
     *
     * @author wang
     * @date  2023/12/07 17:36
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("获取死亡信息分页")
    @GetMapping("/bi/pigdeath/page")
    public CommonResult<Page<PigDeath>> page(PigDeathPageParam pigDeathPageParam) {
        return CommonResult.data(pigDeathService.page(pigDeathPageParam));
    }

    /**
     * 添加死亡信息
     *
     * @author wang
     * @date 2023/12/07 17:36
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("添加死亡信息")
    @CommonLog("添加死亡信息")
    @PostMapping("/bi/pigdeath/add")
    public CommonResult<String> add(@RequestParam(name = "eartag") String eartag,
                                    @RequestParam(name = "unitId") String unitId,
                                    @RequestParam(name = "deathDate") Date deathDate,
                                    @RequestParam(name = "deathReason") String deathReason,
                                    @RequestParam(name = "weight") Double weight,
                                    @RequestParam(name = "deathImg") MultipartFile deathImg,
                                    @RequestParam(name = "remark") String remark,
                                    @RequestParam(name = "orgId") String orgId) throws IOException {
        PigDeathAddParam pigDeathAddParam = new PigDeathAddParam();
        String imageCom = "";
        if (ObjectUtil.isNotEmpty(deathImg)) {
            if (uploadImage.getImageCom(deathImg).equals("上传失败")) {
                TransactionAspectSupport.currentTransactionStatus().setRollbackOnly();
                return new CommonResult(500, "图片上传失败", false);
            }
            imageCom = uploadImage.getImageCom(deathImg);
            pigDeathAddParam.setDeathImg(imageCom);
        }
        pigDeathAddParam.setEartag(eartag);
        pigDeathAddParam.setDeathDate(deathDate);
        pigDeathAddParam.setCreateDate(new Date());
        pigDeathAddParam.setUnitId(unitId);
        pigDeathAddParam.setDeathReason(remark);
        pigDeathAddParam.setOrgId(orgId);
        pigDeathAddParam.setWeight(weight);
        pigDeathAddParam.setDeathReason(deathReason);
        return pigDeathService.add(pigDeathAddParam);
    }

    /**
     * 编辑死亡信息
     *
     * @author wang
     * @date  2023/12/07 17:36
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("编辑死亡信息")
    @CommonLog("编辑死亡信息")
    @PostMapping("/bi/pigdeath/edit")
    public CommonResult<String> edit(@RequestBody @Valid PigDeathEditParam pigDeathEditParam) {
        pigDeathService.edit(pigDeathEditParam);
        return CommonResult.ok();
    }

    /**
     * 删除死亡信息
     *
     * @author wang
     * @date  2023/12/07 17:36
     */
    @ApiOperationSupport(order = 4)
    @ApiOperation("删除死亡信息")
    @CommonLog("删除死亡信息")
    @PostMapping("/bi/pigdeath/delete")
    public CommonResult<String> delete(@RequestBody @Valid @NotEmpty(message = "集合不能为空")
                                                   CommonValidList<PigDeathIdParam> pigDeathIdParamList) {
        pigDeathService.delete(pigDeathIdParamList);
        return CommonResult.ok();
    }

    /**
     * 获取死亡信息详情
     *
     * @author wang
     * @date  2023/12/07 17:36
     */
    @ApiOperationSupport(order = 5)
    @ApiOperation("获取死亡信息详情")
    @GetMapping("/bi/pigdeath/detail")
    public CommonResult<PigDeath> detail(@Valid PigDeathIdParam pigDeathIdParam) {
        return CommonResult.data(pigDeathService.detail(pigDeathIdParam));
    }
}
