/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.bi.pigtransplant.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.modular.bi.pigtransplant.entity.PigTransplant;
import vip.xiaonuo.modular.bi.pigtransplant.mapper.PigTransplantMapper;
import vip.xiaonuo.modular.bi.pigtransplant.param.PigTransplantAddParam;
import vip.xiaonuo.modular.bi.pigtransplant.param.PigTransplantEditParam;
import vip.xiaonuo.modular.bi.pigtransplant.param.PigTransplantIdParam;
import vip.xiaonuo.modular.bi.pigtransplant.param.PigTransplantPageParam;
import vip.xiaonuo.modular.bi.pigtransplant.service.PigTransplantService;

import java.util.List;

/**
 * 转母猪信息Service接口实现类
 *
 * @author wang
 * @date  2023/12/08 10:32
 **/
@Service
public class PigTransplantServiceImpl extends ServiceImpl<PigTransplantMapper, PigTransplant> implements PigTransplantService {

    @Override
    public Page<PigTransplant> page(PigTransplantPageParam pigTransplantPageParam) {
        QueryWrapper<PigTransplant> queryWrapper = new QueryWrapper<>();
        if(ObjectUtil.isNotEmpty(pigTransplantPageParam.getEartag())) {
            queryWrapper.lambda().like(PigTransplant::getEartag, pigTransplantPageParam.getEartag());
        }
        if(ObjectUtil.isNotEmpty(pigTransplantPageParam.getStartTransplantDate()) && ObjectUtil.isNotEmpty(pigTransplantPageParam.getEndTransplantDate())) {
            queryWrapper.lambda().between(PigTransplant::getTransplantDate, pigTransplantPageParam.getStartTransplantDate(), pigTransplantPageParam.getEndTransplantDate());
        }
        if(ObjectUtil.isAllNotEmpty(pigTransplantPageParam.getSortField(), pigTransplantPageParam.getSortOrder())) {
            CommonSortOrderEnum.validate(pigTransplantPageParam.getSortOrder());
            queryWrapper.orderBy(true, pigTransplantPageParam.getSortOrder().equals(CommonSortOrderEnum.ASC.getValue()),
                    StrUtil.toUnderlineCase(pigTransplantPageParam.getSortField()));
        } else {
            queryWrapper.lambda().orderByAsc(PigTransplant::getId);
        }
        return this.page(CommonPageRequest.defaultPage(), queryWrapper);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PigTransplantAddParam pigTransplantAddParam) {
        PigTransplant pigTransplant = BeanUtil.toBean(pigTransplantAddParam, PigTransplant.class);
        this.save(pigTransplant);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(PigTransplantEditParam pigTransplantEditParam) {
        PigTransplant pigTransplant = this.queryEntity(pigTransplantEditParam.getId());
        BeanUtil.copyProperties(pigTransplantEditParam, pigTransplant);
        this.updateById(pigTransplant);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<PigTransplantIdParam> pigTransplantIdParamList) {
        // 执行删除
        this.removeByIds(CollStreamUtil.toList(pigTransplantIdParamList, PigTransplantIdParam::getId));
    }

    @Override
    public PigTransplant detail(PigTransplantIdParam pigTransplantIdParam) {
        return this.queryEntity(pigTransplantIdParam.getId());
    }

    @Override
    public PigTransplant queryEntity(Integer id) {
        PigTransplant pigTransplant = this.getById(id);
        if(ObjectUtil.isEmpty(pigTransplant)) {
            throw new CommonException("转母猪信息不存在，id值为：{}", id);
        }
        return pigTransplant;
    }
}
