/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.modular.warning.warningManager.controller;

import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;
import vip.xiaonuo.biz.modular.user.entity.BizUser;
import vip.xiaonuo.common.pojo.CommonResult;
import vip.xiaonuo.modular.warning.warningManager.entity.vo.WarningManagerVo;
import vip.xiaonuo.modular.warning.warningManager.param.WarningManagerOrgIdParam;
import vip.xiaonuo.modular.warning.warningManager.param.WarningManagerSaveParam;
import vip.xiaonuo.modular.warning.warningManager.param.WarningManagerTypeParam;
import vip.xiaonuo.modular.warning.warningManager.service.WarningManagerService;
import vip.xiaonuo.sys.modular.user.entity.SysUser;


import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

/**
 * 预警负责人控制器
 *
 * @author newspaper
 * @date  2023/12/25 09:36
 */
@Api(tags = "预警负责人控制器")
@ApiSupport(author = "HUIMV", order = 1)
@RestController
@Validated
public class WarningManagerController {

    @Resource
    private WarningManagerService warningManagerService;

    /**
     * 根据类型获取预警负责人
     *
     * @author newspaper
     * @date  2023/12/25 09:36
     */
    @ApiOperationSupport(order = 1)
    @ApiOperation("根据类型获取预警负责人")
    @GetMapping("/warning/warningManager/getManagerByType")
    public CommonResult<WarningManagerVo> getManagerByType(WarningManagerTypeParam warningManagerTypeParam) {
        return CommonResult.data(warningManagerService.getManagerByType(warningManagerTypeParam));
    }

    /**
     * 保存预警负责人
     *
     * @author newspaper
     * @date  2023/12/25 09:36
     */
    @ApiOperationSupport(order = 2)
    @ApiOperation("保存预警负责人")
    @PostMapping("/warning/warningManager/saveManager")
    public CommonResult<String> saveManager(@RequestBody @Valid WarningManagerSaveParam warningManagerSaveParam) {
        warningManagerService.saveManager(warningManagerSaveParam);
        return CommonResult.ok();
    }

    /**
     * 获取人员选择器
     *
     * @author newspaper
     * @date  2023/12/25 09:36
     */
    @ApiOperationSupport(order = 3)
    @ApiOperation("获取人员选择器")
    @PostMapping("/warning/warningManager/userSelector")
    public CommonResult<List<SysUser>> userSelector(@RequestBody @Valid WarningManagerOrgIdParam warningManagerOrgIdParam) {
        return CommonResult.data(warningManagerService.userSelector(warningManagerOrgIdParam));
    }
}
